@@ -5,6 +5,7 @@ __all__ = [
     'ShapDependenceComposite',
     'ShapInteractionsComposite',
     'DecisionTreesComposite',
+    'WhatIfComposite',
 ]
 
 import dash_bootstrap_components as dbc
@@ -50,8 +51,10 @@ class ClassifierModelStatsComposite(ExplainerComponent):
 
         self.hide_title = hide_title
 
+        self.summary = ClassifierModelSummaryComponent(explainer, hide_selector=hide_selector, pos_label=pos_label)
         self.precision = PrecisionComponent(explainer, hide_selector=hide_selector, pos_label=pos_label)
         self.confusionmatrix = ConfusionMatrixComponent(explainer, hide_selector=hide_selector, pos_label=pos_label)
+        self.cumulative_precision = CumulativePrecisionComponent(explainer, hide_selector=hide_selector, pos_label=pos_label)
         self.liftcurve = LiftCurveComponent(explainer, hide_selector=hide_selector, pos_label=pos_label)
         self.classification = ClassificationComponent(explainer, hide_selector=hide_selector, pos_label=pos_label)
         self.rocauc = RocAucComponent(explainer, hide_selector=hide_selector, pos_label=pos_label)
@@ -59,13 +62,13 @@ class ClassifierModelStatsComposite(ExplainerComponent):
 
         self.cutoffpercentile = CutoffPercentileComponent(explainer, hide_selector=hide_selector, pos_label=pos_label)
         self.cutoffconnector = CutoffConnector(self.cutoffpercentile,
-                [self.precision, self.confusionmatrix, self.liftcurve, 
+                [self.summary, self.precision, self.confusionmatrix, self.liftcurve, 
                  self.classification, self.rocauc, self.prauc])
 
         self.register_components(
-            self.precision, self.confusionmatrix, self.liftcurve,
-            self.classification, self.rocauc, self.prauc, self.cutoffpercentile,
-            self.cutoffconnector)
+            self.summary, self.precision, self.confusionmatrix, 
+            self.cumulative_precision, self.liftcurve, self.classification, 
+            self.rocauc, self.prauc, self.cutoffpercentile, self.cutoffconnector)
 
     def layout(self):
         return html.Div([
@@ -81,20 +84,12 @@ class ClassifierModelStatsComposite(ExplainerComponent):
             ]),
             dbc.Row([
                 dbc.Col([
-                    self.precision.layout()
+                    self.summary.layout()
                 ], md=6, align="start"),
                 dbc.Col([
                     self.confusionmatrix.layout()
                 ], md=6, align="start"),              
             ]),
-            dbc.Row([
-                dbc.Col([
-                    self.liftcurve.layout()         
-                ], md=6, align="start"),
-                dbc.Col([
-                    self.classification.layout()
-                ], md=6, align="start"),
-            ]),
             dbc.Row([    
                 dbc.Col([
                     self.rocauc.layout()
@@ -103,6 +98,22 @@ class ClassifierModelStatsComposite(ExplainerComponent):
                     self.prauc.layout()
                 ], md=6),
             ]),
+            dbc.Row([
+                dbc.Col([
+                    self.liftcurve.layout()         
+                ], md=6, align="start"),
+                dbc.Col([
+                    self.classification.layout()
+                ], md=6, align="start"),
+            ]),
+            dbc.Row([
+                dbc.Col([
+                    self.precision.layout()
+                ], md=6, align="start"),
+                dbc.Col([
+                    self.cumulative_precision.layout()
+                ], md=6, align="start"),              
+            ]),  
         ])
 
 
@@ -157,10 +168,10 @@ class RegressionModelStatsComposite(ExplainerComponent):
             ]),
             dbc.Row([
                 dbc.Col([
-                    self.preds_vs_actual.layout()
+                    self.modelsummary.layout()     
                 ], md=6),
                 dbc.Col([
-                    self.modelsummary.layout()     
+                    self.preds_vs_actual.layout()
                 ], md=6),
             ], align="start"),
             dbc.Row([
@@ -222,32 +233,86 @@ class IndividualPredictionsComposite(ExplainerComponent):
             self.pdp, self.contributions_list, self.index_connector)
 
     def layout(self):
-        return html.Div([
+        return dbc.Container([
                 dbc.Row([
                     dbc.Col([
                         self.index.layout()
-                    ]),
+                    ], md=6),
                     dbc.Col([
                         self.summary.layout()
-                    ])
+                    ], md=6)
                 ]),
                 dbc.Row([
                     dbc.Col([
                         self.contributions.layout()
-                    ]),
+                    ], md=6),
                     dbc.Col([
                         self.pdp.layout()
-                    ]),
+                    ], md=6),
                 ]),
                 dbc.Row([
                     dbc.Col([
                         self.contributions_list.layout()
-                    ]),
+                    ], md=6),
                     dbc.Col([
                         html.Div([]),
-                    ]),
+                    ], md=6),
                 ])
-        ])
+        ], fluid=True)
+
+
+class WhatIfComposite(ExplainerComponent):
+    def __init__(self, explainer, title="What if...", name=None,
+                        hide_title=False, hide_selector=True):
+        """Composite for the whatif component:
+
+        Args:
+            explainer (Explainer): explainer object constructed with either
+                        ClassifierExplainer() or RegressionExplainer()
+            title (str, optional): Title of tab or page. Defaults to 
+                        "Individual Predictions".
+            name (str, optional): unique name to add to Component elements. 
+                        If None then random uuid is generated to make sure 
+                        it's unique. Defaults to None.
+            hide_title (bool, optional): hide title. Defaults to False.
+            hide_selector(bool, optional): hide all pos label selectors. Defaults to True.
+        """
+        super().__init__(explainer, title, name)
+
+        self.hide_title = hide_title
+
+        if self.explainer.is_classifier:
+            self.index = ClassifierRandomIndexComponent(
+                            explainer, hide_selector=hide_selector)
+        elif self.explainer.is_regression:
+            self.index = RegressionRandomIndexComponent(explainer)
+
+        self.whatif = WhatIfComponent(explainer, 
+                            hide_title=True, hide_index=True, hide_selector=hide_selector)
+
+        self.index_connector = IndexConnector(self.index, [self.whatif])
+
+        self.register_components(self.index, self.whatif, self.index_connector)
+
+    def layout(self):
+        return dbc.Container([
+                dbc.Row([
+                    make_hideable(
+                        dbc.Col([
+                            html.H1("What if...")
+                        ]), hide=self.hide_title),
+                ]),
+                dbc.Row([
+                    dbc.Col([
+                        self.index.layout()
+                    ]),
+                ]),
+                dbc.Row([
+                    dbc.Col([
+                        self.whatif.layout()
+                    ]),
+                ]),
+        ], fluid=True)
 
 
 class ShapDependenceComposite(ExplainerComponent):
@@ -325,16 +390,16 @@ class ShapInteractionsComposite(ExplainerComponent):
             self.interaction_summary, self.interaction_dependence, self.connector)
         
     def layout(self):
-        return html.Div([
+        return dbc.Container([
                 dbc.Row([
                     dbc.Col([
                         self.interaction_summary.layout()
-                    ], width=6),
+                    ], md=6),
                     dbc.Col([
                         self.interaction_dependence.layout()
-                    ], width=6),
+                    ], md=6),
                 ]) 
-        ])
+        ], fluid=True)
 
 
 class DecisionTreesComposite(ExplainerComponent):
@@ -376,19 +441,6 @@ class DecisionTreesComposite(ExplainerComponent):
         self.register_components(self.index, self.trees, 
                 self.decisionpath_table, self.decisionpath_graph, 
                 self.index_connector, self.highlight_connector)
-
-        # elif explainer.is_regression:
-        #     self.index = RegressionRandomIndexComponent(explainer)
-        #     self.decisionpath_graph = DecisionPathGraphComponent(explainer, hide_selector=hide_selector)# DummyComponent()
-        #     self.index_connector = IndexConnector(self.index, 
-        #             [self.trees, self.decisionpath_table])
-        #     self.highlight_connector = HighlightConnector(
-        #             self.trees, [self.decisionpath_table, self.decisionpath_graph])
-
-        #     self.register_components(
-        #             self.index, self.trees, self.decisionpath_table,
-        #             self.decisionpath_graph, self.index_connector, 
-        #             self.highlight_connector)
         
     def layout(self):
         return html.Div([
