@@ -13,6 +13,7 @@ import requests
 
 import shortuuid
 import dash
+import dash_auth
 import dash_core_components as dcc
 import dash_html_components as html
 import dash_bootstrap_components as dbc
@@ -21,6 +22,7 @@ from jupyter_dash import JupyterDash
 
 import plotly.io as pio
 
+
 from .dashboard_components import *
 from .dashboard_tabs import *
 from .explainers import BaseExplainer
@@ -249,9 +251,12 @@ class ExplainerDashboard:
                  external_stylesheets=None,
                  server=True,
                  url_base_pathname=None,
+                 responsive=True,
+                 logins=None,
                  importances=True,
                  model_summary=True,
                  contributions=True,
+                 whatif=True,
                  shap_dependence=True,
                  shap_interaction=True,
                  decision_trees=True,
@@ -311,9 +316,21 @@ class ExplainerDashboard:
             height(int, optional): height of notebookn output cell in pixels, defaults to 800.
             external_stylesheets(list, optional): attach dbc themes e.g. 
                 `external_stylesheets=[dbc.themes.FLATLY]`. 
+            server (Flask instance or bool): either an instance of an existing Flask
+                server to tie the dashboard to, or True in which case a new Flask
+                server is created. 
+            url_base_pathname (str): url_base_pathname for dashboard, 
+                e.g. "/dashboard". Defaults to None.
+            responsive (bool):  make layout responsive to viewport size 
+                (i.e. reorganize bootstrap columns on small devices). Set to False
+                when e.g. testing with a headless browser. Defaults to True.
+            logins (list of lists): list of (hardcoded) logins, e.g. 
+                [['login1', 'password1'], ['login2', 'password2']]. 
+                Defaults to None (no login required)
             importances(bool, optional): include ImportancesTab, defaults to True.
             model_summary(bool, optional): include ModelSummaryTab, defaults to True.
             contributions(bool, optional): include ContributionsTab, defaults to True.
+            whatif (bool, optional): include WhatIfTab, defaults to True.
             shap_dependence(bool, optional): include ShapDependenceTab, defaults to True.
             shap_interaction(bool, optional): include InteractionsTab if model allows it, defaults to True.
             decision_trees(bool, optional): include DecisionTreesTab if model allows it, defaults to True.
@@ -327,11 +344,28 @@ class ExplainerDashboard:
             hide_header, header_hide_title, header_hide_selector
         self.external_stylesheets = external_stylesheets
         self.server, self.url_base_pathname = server, url_base_pathname
-        
+        self.responsive = responsive
+
+        try:
+            ipython_kernel = str(get_ipython())
+            self.is_notebook = True
+            self.is_colab = True if 'google.colab' in ipython_kernel else False
+        except:
+            self.is_notebook, self.is_colab = False, False
+
+        if self.mode == 'dash' and self.is_colab:
+            print("Detected google colab environment, setting mode='external'", flush=True)
+            self.mode = 'external'
+        elif self.mode == 'dash' and self.is_notebook:
+            print("Detected notebook environment, consider setting "
+                    "mode='external', mode='inline' or mode='jupyterlab'...", flush=True)
+
         self.app = self._get_dash_app()
+        if logins is not None:
+            self.auth = dash_auth.BasicAuth(self.app, logins)
         self.app.title = title
 
-        assert isinstance(explainer, BaseExplainer), \
+        assert 'BaseExplainer' in str(explainer.__class__.mro()), \
             ("explainer should be an instance of BaseExplainer, such as "
             "ClassifierExplainer or RegressionExplainer!")
         
@@ -352,6 +386,8 @@ class ExplainerDashboard:
                 tabs.append(ModelSummaryTab)
             if contributions:
                 tabs.append(ContributionsTab)
+            if whatif:
+                tabs.append(WhatIfTab)
             if shap_dependence:
                 tabs.append(ShapDependenceTab)
             if shap_interaction:
@@ -383,7 +419,7 @@ class ExplainerDashboard:
 
         print("Calculating dependencies...", flush=True)  
         explainer_layout.calculate_dependencies()
-        print("registering callbacks...", flush=True)
+        print("Registering callbacks...", flush=True)
         explainer_layout.register_callbacks(self.app)
 
     def _convert_str_tabs(self, component):
@@ -403,17 +439,23 @@ class ExplainerDashboard:
         return component
 
     def _get_dash_app(self):
+        if self.responsive:
+            meta_tags = [{'name': 'viewport', 'content': 'width=device-width, initial-scale=1.0, maximum-scale=1.2, minimum-scale=0.5,'}]
+        else:
+            meta_tags = None
         if self.mode=="dash":
             if self.external_stylesheets is not None:
                 app = dash.Dash(server=self.server, 
                                 external_stylesheets=self.external_stylesheets, 
                                 assets_url_path="", 
-                                url_base_pathname=self.url_base_pathname)
+                                url_base_pathname=self.url_base_pathname,
+                                meta_tags=meta_tags)
                 app.config['suppress_callback_exceptions'] = True
             else:
                 app = dash.Dash(__name__, 
                                 server=self.server, 
-                                url_base_pathname=self.url_base_pathname)
+                                url_base_pathname=self.url_base_pathname,
+                                meta_tags=meta_tags)
                 app.config['suppress_callback_exceptions'] = True
                 app.css.config.serve_locally = True
                 app.scripts.config.serve_locally = True
@@ -422,7 +464,8 @@ class ExplainerDashboard:
             if self.external_stylesheets is not None:
                 app = JupyterDash(
                             external_stylesheets=self.external_stylesheets, 
-                            assets_url_path="")
+                            assets_url_path="",
+                            meta_tags=meta_tags)
             else:
                 app = JupyterDash(__name__)
             return app
