@@ -1,3 +1,23 @@
+__all__= [
+    'plotly_contribution_plot',
+    'plotly_precision_plot',
+    'plotly_classification_plot',
+    'plotly_lift_curve',
+    'plotly_cumulative_precision_plot',
+    'plotly_dependence_plot',
+    'plotly_shap_violin_plot',
+    'plotly_pdp',
+    'plotly_importances_plot',
+    'plotly_confusion_matrix',
+    'plotly_roc_auc_curve',
+    'plotly_pr_auc_curve',
+    'plotly_shap_scatter_plot',
+    'plotly_predicted_vs_actual',
+    'plotly_plot_residuals',
+    'plotly_residuals_vs_col',
+    'plotly_rf_trees',
+    'plotly_xgboost_trees']
+
 import numpy as np
 import pandas as pd
 from pandas.api.types import is_numeric_dtype, is_string_dtype
@@ -10,7 +30,7 @@ from sklearn.metrics import (classification_report, confusion_matrix,
                              roc_auc_score, average_precision_score)
 
 
-def plotly_contribution_plot(contrib_df, target="target", 
+def plotly_contribution_plot(contrib_df, target="", 
                          model_output="raw", higher_is_better=False,
                          include_base_value=True, include_prediction=True, 
                          orientation='vertical', round=2, units=""):
@@ -70,10 +90,10 @@ def plotly_contribution_plot(contrib_df, target="target",
         hover_text=[f"{col}=?<BR>{'+' if contrib>0 else ''}{contrib} {units}"  
                   for col, contrib in zip(cols, contribs)]
 
-    fill_color_up='rgba(55, 128, 191, 0.7)' if higher_is_better else 'rgba(219, 64, 82, 0.7)'
-    fill_color_down='rgba(219, 64, 82, 0.7)' if higher_is_better else 'rgba(55, 128, 191, 0.7)'
-    line_color_up='rgba(55, 128, 191, 1.0)' if higher_is_better else 'rgba(219, 64, 82, 1.0)'
-    line_color_down='rgba(219, 64, 82, 1.0)' if higher_is_better else 'rgba(55, 128, 191, 1.0)'
+    fill_color_up = 'rgba(55, 128, 191, 0.7)' if higher_is_better else 'rgba(219, 64, 82, 0.7)'
+    fill_color_down = 'rgba(219, 64, 82, 0.7)' if higher_is_better else 'rgba(55, 128, 191, 0.7)'
+    line_color_up = 'rgba(55, 128, 191, 1.0)' if higher_is_better else 'rgba(219, 64, 82, 1.0)'
+    line_color_down = 'rgba(219, 64, 82, 1.0)' if higher_is_better else 'rgba(55, 128, 191, 1.0)'
 
     fill_colors = [fill_color_up if y > 0 else fill_color_down for y in contribs]
     line_colors = [line_color_up if y > 0 else line_color_down for y in contribs]
@@ -163,7 +183,6 @@ def plotly_contribution_plot(contrib_df, target="target",
     return fig
 
 
-
 def plotly_precision_plot(precision_df, cutoff=None, labels=None, pos_label=None):
     """
     returns a plotly figure with bar plots for counts of observations for a 
@@ -286,9 +305,9 @@ def plotly_classification_plot(pred_probas, targets, labels=None, cutoff=0.5,
     
     fig = go.Figure()
     for i, label in enumerate(labels):
-        text = [f"<b>{sum(below_threshold[1]==i)}</b> ({np.round(100*np.mean(below_threshold[1]==i), 2)}%)",
-                f"<b>{sum(above_threshold[1]==i)}</b> ({np.round(100*np.mean(above_threshold[1]==i), 2)}%)", 
-                f"<b>{sum(targets==i)}</b> ({np.round(100*np.mean(targets==i), 2)}%)"]
+        text = [f"<b>{sum(below_threshold[1]==i)}</b><br>({np.round(100*np.mean(below_threshold[1]==i), 1)}%)",
+                f"<b>{sum(above_threshold[1]==i)}</b><br>({np.round(100*np.mean(above_threshold[1]==i), 1)}%)", 
+                f"<b>{sum(targets==i)}</b><br>({np.round(100*np.mean(targets==i), 1)}%)"]
         if percentage:
             fig.add_trace(go.Bar(
                 x=x, 
@@ -818,7 +837,7 @@ def plotly_shap_violin_plot(X, shap_values, col_name, color_col=None, points=Fal
 def plotly_pdp(pdp_result, 
                display_index=None, index_feature_value=None, index_prediction=None,
                absolute=True, plot_lines=True, num_grid_lines=100, feature_name=None,
-               round=2, units=""):
+               round=2, target="", units=""):
     """
     display_index: display the pdp of particular index
     index_feature_value: the actual feature value of the index to be highlighted
@@ -872,7 +891,8 @@ def plotly_pdp(pdp_result,
 
     layout = go.Layout(title = f'pdp plot for {feature_name}',
                         plot_bgcolor = '#fff',
-                        yaxis=dict(title=units))
+                        yaxis=dict(title=f"Predicted {target}{f' ({units})' if units else ''}"),
+                        xaxis=dict(title=feature_name))
 
     fig = go.Figure(data=data, layout=layout)
 
@@ -935,7 +955,8 @@ def plotly_pdp(pdp_result,
     return fig
 
 
-def plotly_importances_plot(importance_df, descriptions=None, round=3, units="", title=None, xaxis_title=None):
+def plotly_importances_plot(importance_df, descriptions=None, round=3, 
+            target="target" , units="", title=None, xaxis_title=None):
     
     importance_name = importance_df.columns[1] # can be "MEAN_ABS_SHAP", "Permutation Importance", etc
     if title is None:
@@ -985,106 +1006,15 @@ def plotly_importances_plot(importance_df, descriptions=None, round=3, units="",
     return fig
 
 
-def plotly_tree_predictions(model, observation, y=None, highlight_tree=None, round=2, pos_label=1, units=""):
-    """
-    returns a plot with all the individual predictions of the 
-    DecisionTrees that make up the RandomForest.
-    """
-    assert (str(type(model)).endswith("RandomForestClassifier'>") 
-            or str(type(model)).endswith("RandomForestRegressor'>")), \
-        f"model is of type {type(model)}, but should be either RandomForestClassifier or RandomForestRegressor"
-    
-    colors = ['blue'] * len(model.estimators_) 
-    if highlight_tree is not None:
-        assert highlight_tree >= 0 and highlight_tree <= len(model.estimators_), \
-            f"{highlight_tree} is out of range (0, {len(model.estimators_)})"
-        colors[highlight_tree] = 'red'
-        
-    if model.estimators_[0].classes_[0] is not None: #if classifier
-        preds_df = (
-            pd.DataFrame({
-                'model' : range(len(model.estimators_)), 
-                'prediction' : [
-                        np.round(100*m.predict_proba(observation)[0, pos_label], round) 
-                                    for m in model.estimators_],
-                'color' : colors
-            })
-            .sort_values('prediction')\
-            .reset_index(drop=True))
-    else:
-        preds_df = (
-            pd.DataFrame({
-                'model' : range(len(model.estimators_)), 
-                'prediction' : [np.round(m.predict(observation)[0] , round)
-                                    for m in model.estimators_],
-                'color' : colors
-            })
-            .sort_values('prediction')\
-            .reset_index(drop=True))
-      
-    trace0 = go.Bar(x=preds_df.index, 
-                    y=preds_df.prediction, 
-                    marker_color=preds_df.color,
-                    text=[f"tree no {t}:<br> prediction={p}<br> click for detailed info"
-                             for (t, p) in zip(preds_df.model.values, preds_df.prediction.values)],
-                    hoverinfo="text")
-    
-    layout = go.Layout(
-                title='individual predictions trees',
-                plot_bgcolor = '#fff',
-                yaxis=dict(title=units)
-            )
-    fig = go.Figure(data = [trace0], layout=layout)
-    shapes = [dict(
-                type='line',
-                xref='x', yref='y',
-                x0=0, x1=preds_df.model.max(), 
-                y0=preds_df.prediction.mean(), y1=preds_df.prediction.mean(),
-                line=dict(
-                    color="darkslategray",
-                    width=4,
-                    dash="dot"),
-                )]
-    
-    annotations = [go.layout.Annotation(
-        x=preds_df.model.mean(), 
-        y=preds_df.prediction.mean(),
-        text=f"Average prediction = {np.round(preds_df.prediction.mean(),2)}",
-        bgcolor="white")]
-
-    if y is not None:
-        shapes.append(dict(
-                type='line',
-                xref='x', yref='y',
-                x0=0, x1=preds_df.model.max(), 
-                y0=y, y1=y,
-                line=dict(
-                    color="red",
-                    width=4,
-                    dash="dashdot"),
-                ))
-        annotations.append(go.layout.Annotation(
-            x=preds_df.model.mean(), y=y, text=f"y={y}", bgcolor="red"))
-
-    fig.update_layout(shapes=shapes)
-    fig.update_layout(annotations=annotations)
-    
-    return fig 
-
-
-
-def plotly_confusion_matrix(y_true, y_preds, labels = None, normalized=True):
+def plotly_confusion_matrix(y_true, y_preds, labels = None, percentage=True):
 
     cm = confusion_matrix(y_true, y_preds)
+    cm_normalized = np.round(100*cm / cm.sum(), 1)
 
     if labels is None:
         labels = [str(i) for i in range(cm.shape[0])] 
 
-    if normalized:
-        cm = np.round(100*cm / cm.sum(),1)
-        zmax = 100
-    else:
-        zmax = len(y_true)
+    zmax = len(y_true)
         
     data=[go.Heatmap(
                         z=cm,
@@ -1105,16 +1035,25 @@ def plotly_confusion_matrix(y_true, y_preds, labels = None, normalized=True):
     annotations = []
     for x in range(cm.shape[0]):
         for y in range(cm.shape[1]):
-            text= str(cm[x,y]) + '%' if normalized else str(cm[x,y])
-            annotations.append(
+            top_text = f"{cm_normalized[x, y]}%" if percentage else f"{cm[x, y]}"
+            bottom_text = f"{cm_normalized[x, y]}%" if not percentage else f"{cm[x, y]}" 
+            annotations.extend([
                 go.layout.Annotation(
                     x=fig.data[0].x[y], 
                     y=fig.data[0].y[x], 
-                    text=text, 
+                    text=top_text, 
                     showarrow=False,
                     font=dict(size=20)
-                )
+                ),
+                go.layout.Annotation(
+                    x=fig.data[0].x[y], 
+                    y=fig.data[0].y[x], 
+                    text=f" <br> <br> <br>({bottom_text})", 
+                    showarrow=False,
+                    font=dict(size=12)
+                )]
             )
+                
 
     fig.update_layout(annotations=annotations)  
     return fig
@@ -1269,7 +1208,8 @@ def plotly_pr_auc_curve(true_y, pred_probas, cutoff=None):
     return fig
 
 
-def plotly_shap_scatter_plot(shap_values, X, display_columns, title="Shap values", idxs=None):
+def plotly_shap_scatter_plot(shap_values, X, display_columns, title="Shap values", 
+                idxs=None, highlight_index=None, na_fill=-999):
     
     if idxs is not None:
         assert len(idxs)==X.shape[0]
@@ -1277,6 +1217,18 @@ def plotly_shap_scatter_plot(shap_values, X, display_columns, title="Shap values
     else:
         idxs = np.array([str(i) for i in range(X.shape[0])])
         
+    if highlight_index is not None:
+        if isinstance(highlight_index, int):
+            assert highlight_index >=0 and highlight_index < len(X), \
+            "if highlight_index is int, then should be between 0 and {len(X)}!"
+            highlight_idx = highlight_index
+            highlight_index = idxs[highlight_idx]
+        elif isinstance(highlight_index, str):
+            assert str(highlight_index) in idxs, f"{highlight_index} not found in idxs!"
+            highlight_idx = np.where(idxs == str(highlight_index))[0].item()
+        else:
+            raise ValueError("Please pass either int or str highlight_index!")
+        
     # make sure that columns are actually in X:
     display_columns = [col for col in display_columns if col in X.columns.tolist()]    
     shap_df = pd.DataFrame(shap_values, columns=X.columns)
@@ -1316,7 +1268,7 @@ def plotly_shap_scatter_plot(shap_values, X, display_columns, title="Shap values
                                   mode='markers',
                                   marker=dict(
                                       size=5,
-                                      color=X[col].replace({-999:np.nan}),
+                                      color=X[col].replace({na_fill:np.nan}),
                                       colorscale='Bluered',
                                       showscale=True,
                                       opacity=0.3,
@@ -1332,6 +1284,26 @@ def plotly_shap_scatter_plot(shap_values, X, display_columns, title="Shap values
                                       for i, shap, value in zip(idxs, shap_df[col], X[col].replace({-999:np.nan}))],
                                 ),
                      row=i+1, col=1);
+        if highlight_index is not None:
+            fig.add_trace(
+            go.Scatter(
+                x=[shap_df[col].iloc[highlight_idx]], 
+                y=[0], 
+                mode='markers',
+                marker=dict(
+                    color='LightSkyBlue',
+                    size=20,
+                    opacity=0.5,
+                    line=dict(
+                        color='MediumPurple',
+                        width=4
+                    )
+                ),
+                name = f"index {highlight_index}",
+                text=f"index={highlight_index}<br>{col}={X[col].iloc[highlight_idx]}<br>shap={shap_df[col].iloc[highlight_idx]}",
+                hoverinfo="text",
+                showlegend=False,
+            ), row=i+1, col=1)
         fig.update_xaxes(showgrid=False, zeroline=False, 
                          range=[min_shap, max_shap], row=i+1, col=1)
         fig.update_yaxes(showgrid=False, zeroline=False, 
@@ -1343,7 +1315,7 @@ def plotly_shap_scatter_plot(shap_values, X, display_columns, title="Shap values
                                 l=50,
                                 r=50,
                                 b=50,
-                                t=50,
+                                t=100,
                                 pad=4
                             ),
                       hovermode='closest',
@@ -1351,7 +1323,8 @@ def plotly_shap_scatter_plot(shap_values, X, display_columns, title="Shap values
     return fig
 
 
-def plotly_predicted_vs_actual(y, preds, units="", round=2, logs=False, log_x=False, log_y=False, idxs=None):
+def plotly_predicted_vs_actual(y, preds, target="" , units="", round=2, 
+            logs=False, log_x=False, log_y=False, idxs=None):
     
     if idxs is not None:
         assert len(idxs)==len(preds)
@@ -1368,7 +1341,7 @@ def plotly_predicted_vs_actual(y, preds, units="", round=2, logs=False, log_x=Fa
         x = y,
         y = preds,
         mode='markers', 
-        name='predicted',
+        name=f'predicted {target}' + f" ({units})" if units else "",
         text=marker_text,
         hoverinfo="text",
     )
@@ -1378,19 +1351,19 @@ def plotly_predicted_vs_actual(y, preds, units="", round=2, logs=False, log_x=Fa
         x = sorted_y,
         y = sorted_y,
         mode='lines', 
-        name=f'actual {units}',
+        name=f"actual {target}" + f" ({units})" if units else "",
         hoverinfo="none",
     )
     
     data = [trace0, trace1]
     
     layout = go.Layout(
-        title='Predicted vs Actual',
+        title=f"Predicted {target} vs Actual {target}",
         yaxis=dict(
-            title=f'Predicted {units}' 
+            title=f"Predicted {target}" + f" ({units})" if units else "",
         ),
         xaxis=dict(
-            title=f'Actual {units}' 
+            title=f"Actual {target}" + f" ({units})" if units else "",
         ),
         plot_bgcolor = '#fff',
         hovermode = 'closest',
@@ -1406,7 +1379,7 @@ def plotly_predicted_vs_actual(y, preds, units="", round=2, logs=False, log_x=Fa
     return fig
 
 
-def plotly_plot_residuals(y, preds, vs_actual=False, units="", residuals='difference', round=2, idxs=None):
+def plotly_plot_residuals(y, preds, vs_actual=False, target="", units="", residuals='difference', round=2, idxs=None):
     """generates a residual plot
 
     Args:
@@ -1414,6 +1387,7 @@ def plotly_plot_residuals(y, preds, vs_actual=False, units="", residuals='differ
         preds (np.array, pd.Series): Predictions
         vs_actual (bool, optional): Put actual values (y) on the x-axis. 
                     Defaults to False (i.e. preds on the x-axis)
+        target (str, optional): name of the target variable. Defaults to ""
         units (str, optional): units of the axis. Defaults to "".
         residuals (str, {'difference', 'ratio', 'log-ratio'} optional): 
                     How to calcualte residuals. Defaults to 'difference'.
@@ -1463,20 +1437,22 @@ def plotly_plot_residuals(y, preds, vs_actual=False, units="", residuals='differ
         x=y if vs_actual else preds, 
         y=np.ones(len(preds)) if residuals=='ratio' else np.zeros(len(preds)),
         mode='lines', 
-        name=f'Actual {units}' if vs_actual else f'Predicted {units}',
+        name=(f"Actual {target}" + f" ({units})" if units else "") if vs_actual \
+                else (f"Predicted {target}" + f" ({units})" if units else ""),
         hoverinfo="none",
     )
     
     data = [trace0, trace1]
     
     layout = go.Layout(
-        title=f"Residuals vs {'actual' if vs_actual else 'predicted'}",
+        title=f"Residuals vs {'actual' if vs_actual else 'predicted'} {target}",
         yaxis=dict(
             title=residuals_name
         ),
         
         xaxis=dict(
-            title=f'Actual {units}' if vs_actual else f'Predicted {units}' 
+            title=(f"Actual {target}" + f" ({units})" if units else "") if vs_actual \
+                else (f"Predicted {target}" + f" ({units})" if units else "")
         ),
         plot_bgcolor = '#fff',
         hovermode = 'closest',
@@ -1609,7 +1585,104 @@ def plotly_residuals_vs_col(y, preds, col, col_name=None, residuals='difference'
         return fig
 
 
-def plotly_xgboost_trees(xgboost_preds_df, highlight_tree=None, y=None, round=2,  pos_label=1, units=""):
+def plotly_rf_trees(model, observation, y=None, highlight_tree=None, 
+            round=2, pos_label=1, target="", units=""):
+    """
+    returns a plot with all the individual predictions of the 
+    DecisionTrees that make up the RandomForest.
+    """
+    assert (str(type(model)).endswith("RandomForestClassifier'>") 
+            or str(type(model)).endswith("RandomForestRegressor'>")), \
+        f"model is of type {type(model)}, but should be either RandomForestClassifier or RandomForestRegressor"
+    
+    colors = ['blue'] * len(model.estimators_) 
+    if highlight_tree is not None:
+        assert highlight_tree >= 0 and highlight_tree <= len(model.estimators_), \
+            f"{highlight_tree} is out of range (0, {len(model.estimators_)})"
+        colors[highlight_tree] = 'red'
+        
+    if model.estimators_[0].classes_[0] is not None: #if classifier
+        preds_df = (
+            pd.DataFrame({
+                'model' : range(len(model.estimators_)), 
+                'prediction' : [
+                        np.round(100*m.predict_proba(observation)[0, pos_label], round) 
+                                    for m in model.estimators_],
+                'color' : colors
+            })
+            .sort_values('prediction')\
+            .reset_index(drop=True))
+    else:
+        preds_df = (
+            pd.DataFrame({
+                'model' : range(len(model.estimators_)), 
+                'prediction' : [np.round(m.predict(observation)[0] , round)
+                                    for m in model.estimators_],
+                'color' : colors
+            })
+            .sort_values('prediction')\
+            .reset_index(drop=True))
+      
+    trace0 = go.Bar(x=preds_df.index, 
+                    y=preds_df.prediction, 
+                    marker_color=preds_df.color,
+                    text=[f"tree no {t}:<br> prediction={p}<br> click for detailed info"
+                             for (t, p) in zip(preds_df.model.values, preds_df.prediction.values)],
+                    hoverinfo="text")
+    
+    if target:
+        title = f"Individual RandomForest decision trees predicting {target}"
+        yaxis_title = f"Predicted {target} {f'({units})' if units else ''}"
+    else:
+        title = f"Individual RandomForest decision trees"
+        yaxis_title = f"Predicted outcome ({units})" if units else "Predicted outcome"
+
+    layout = go.Layout(
+                title=title,
+                plot_bgcolor = '#fff',
+                yaxis=dict(title=yaxis_title),
+                xaxis=dict(title="decision trees (sorted by prediction")
+            )
+    fig = go.Figure(data = [trace0], layout=layout)
+    shapes = [dict(
+                type='line',
+                xref='x', yref='y',
+                x0=0, x1=preds_df.model.max(), 
+                y0=preds_df.prediction.mean(), y1=preds_df.prediction.mean(),
+                line=dict(
+                    color="darkslategray",
+                    width=4,
+                    dash="dot"),
+                )]
+    
+    annotations = [go.layout.Annotation(
+        x=preds_df.model.mean(), 
+        y=preds_df.prediction.mean(),
+        text=f"Average prediction = {np.round(preds_df.prediction.mean(),2)}",
+        bgcolor="white")]
+
+    if y is not None:
+        shapes.append(dict(
+                type='line',
+                xref='x', yref='y',
+                x0=0, x1=preds_df.model.max(), 
+                y0=y, y1=y,
+                line=dict(
+                    color="red",
+                    width=4,
+                    dash="dashdot"),
+                ))
+        annotations.append(go.layout.Annotation(
+            x=preds_df.model.mean(), y=y, text=f"y={y}", bgcolor="red"))
+
+    fig.update_layout(shapes=shapes)
+    fig.update_layout(annotations=annotations)
+    
+    return fig 
+
+
+def plotly_xgboost_trees(xgboost_preds_df, highlight_tree=None, y=None, round=2,  
+                            pos_label=1, target="", units=""):
     """
     returns a plot with all the individual predictions of the 
     DecisionTrees that make up the RandomForest.
@@ -1694,11 +1767,19 @@ def plotly_xgboost_trees(xgboost_preds_df, highlight_tree=None, y=None, round=2,
                     ),
                    )
     
+    if target:
+        title = f"Individual xgboost decision trees predicting {target}"
+        yaxis_title = f"Predicted {target} {f'({units})' if units else ''}"
+    else:
+        title = f"Individual xgboost decision trees"
+        yaxis_title = f"Predicted outcome ({units})" if units else "Predicted outcome"
+
     layout = go.Layout(
-                title='individual xgboost prediction trees',
+                title=title,
                 barmode='stack',
                 plot_bgcolor = '#fff',
-                yaxis=dict(title=units)
+                yaxis=dict(title=yaxis_title),
+                xaxis=dict(title="decision trees")
             )
     
     fig = go.Figure(data = [trace0, trace1], layout=layout)
