@@ -28,7 +28,7 @@ class RegressionBaseExplainerTests(unittest.TestCase):
                             model, X_test, y_test, r2_score,
                             shap='tree',
                             cats=['Sex', 'Cabin', 'Embarked'],
-                            idxs=test_names)
+                            idxs=test_names, target='Fare', units='$')
 
     def test_explainer_len(self):
         self.assertEqual(len(self.explainer), self.test_len)
@@ -98,6 +98,8 @@ class RegressionBaseExplainerTests(unittest.TestCase):
         self.assertIsInstance(self.explainer.contrib_df(0, sort='high-to-low'), pd.DataFrame)
         self.assertIsInstance(self.explainer.contrib_df(0, sort='low-to-high'), pd.DataFrame)
         self.assertIsInstance(self.explainer.contrib_df(0, sort='importance'), pd.DataFrame)
+        self.assertIsInstance(self.explainer.contrib_df(X_row=self.explainer.X.iloc[[0]]), pd.DataFrame)
+        self.assertIsInstance(self.explainer.contrib_df(X_row=self.explainer.X_cats.iloc[[0]]), pd.DataFrame)
 
 
     def test_contrib_summary_df(self):
@@ -108,6 +110,9 @@ class RegressionBaseExplainerTests(unittest.TestCase):
         self.assertIsInstance(self.explainer.contrib_summary_df(0, sort='high-to-low'), pd.DataFrame)
         self.assertIsInstance(self.explainer.contrib_summary_df(0, sort='low-to-high'), pd.DataFrame)
         self.assertIsInstance(self.explainer.contrib_summary_df(0, sort='importance'), pd.DataFrame)
+        self.assertIsInstance(self.explainer.contrib_summary_df(X_row=self.explainer.X.iloc[[0]]), pd.DataFrame)
+        self.assertIsInstance(self.explainer.contrib_summary_df(X_row=self.explainer.X_cats.iloc[[0]]), pd.DataFrame)
+
 
     def test_shap_base_value(self):
         self.assertIsInstance(self.explainer.shap_base_value, (np.floating, float))
@@ -142,6 +147,8 @@ class RegressionBaseExplainerTests(unittest.TestCase):
         self.assertIsInstance(self.explainer.get_pdp_result("Sex"), pdpbox.pdp.PDPIsolate)
         self.assertIsInstance(self.explainer.get_pdp_result("Age", index=0), pdpbox.pdp.PDPIsolate)
         self.assertIsInstance(self.explainer.get_pdp_result("Sex", index=0), pdpbox.pdp.PDPIsolate)
+        self.assertIsInstance(self.explainer.get_pdp_result("Age", X_row=self.explainer.X.iloc[[0]]), pdpbox.pdp.PDPIsolate)
+        self.assertIsInstance(self.explainer.get_pdp_result("Sex", X_row=self.explainer.X_cats.iloc[[0]]), pdpbox.pdp.PDPIsolate)
 
     def test_get_dfs(self):
         cols_df, shap_df, contribs_df = self.explainer.get_dfs()
@@ -242,17 +249,42 @@ class RegressionBaseExplainerTests(unittest.TestCase):
         fig = self.explainer.plot_shap_dependence("Sex", highlight_index=0)
         self.assertIsInstance(fig, go.Figure)
 
+    def test_plot_shap_contributions(self):
+        fig = self.explainer.plot_shap_contributions(0)
+        self.assertIsInstance(fig, go.Figure)
+
+        fig = self.explainer.plot_shap_contributions(0, cats=False)
+        self.assertIsInstance(fig, go.Figure)
+
+        fig = self.explainer.plot_shap_contributions(0, topx=3)
+        self.assertIsInstance(fig, go.Figure)
+
+        fig = self.explainer.plot_shap_contributions(0, sort='high-to-low')
+        self.assertIsInstance(fig, go.Figure)
+
+        fig = self.explainer.plot_shap_contributions(0, sort='low-to-high')
+        self.assertIsInstance(fig, go.Figure)
+
+        fig = self.explainer.plot_shap_contributions(0, sort='importance')
+        self.assertIsInstance(fig, go.Figure)
+
+        fig = self.explainer.plot_shap_contributions(X_row=self.explainer.X.iloc[[0]])
+        self.assertIsInstance(fig, go.Figure)
+
+        fig = self.explainer.plot_shap_contributions(X_row=self.explainer.X_cats.iloc[[0]])
+        self.assertIsInstance(fig, go.Figure)
+
     def test_plot_shap_interaction(self):
-        fig = self.explainer.plot_shap_dependence("Age", "Sex_female")
+        fig = self.explainer.plot_shap_interaction("Age", "Sex_female")
         self.assertIsInstance(fig, go.Figure)
 
-        fig = self.explainer.plot_shap_dependence("Sex_female", "Age")
+        fig = self.explainer.plot_shap_interaction("Sex_female", "Age")
         self.assertIsInstance(fig, go.Figure)
 
-        fig = self.explainer.plot_shap_dependence("Sex", "Age")
+        fig = self.explainer.plot_shap_interaction("Sex", "Age")
         self.assertIsInstance(fig, go.Figure)
 
-        fig = self.explainer.plot_shap_dependence("Age", "Sex_female", highlight_index=0)
+        fig = self.explainer.plot_shap_interaction("Age", "Sex_female", highlight_index=0)
         self.assertIsInstance(fig, go.Figure)
 
     def test_plot_pdp(self):
@@ -268,6 +300,12 @@ class RegressionBaseExplainerTests(unittest.TestCase):
         fig = self.explainer.plot_pdp("Age", index=0)
         self.assertIsInstance(fig, go.Figure)
 
+        fig = self.explainer.plot_pdp("Age", X_row=self.explainer.X.iloc[[0]])
+        self.assertIsInstance(fig, go.Figure)
+
+        fig = self.explainer.plot_pdp("Age", X_row=self.explainer.X_cats.iloc[[0]])
+        self.assertIsInstance(fig, go.Figure)
+
 
 
 
